
TACHYON
[~


{
/**************************************************************************
 * Copyright [c] <2012-2013>  - "TACHYON Forth" for the Parallax Propeller
 *                              Peter Jakacki, Brisbane, Australia
 *
 *           [c] <2013>       - Tachyon Forth source code for XRTC
 *                              Brian Riley, Underhill Ctr., VT, USA
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 *     The above copyright notice and this permission notice shall be 
 *     included in all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
 *   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 *   BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN 
 *   ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 *   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 *   SOFTWARE.
 *
 **************************************************************************/
}

{
						DSRTC.fth
						
	UPDATED 20130908.1830 - BBR - almost a complete rewrite of chip register i/o
								- verified/corrected all words are stack neutral
								- added more print formats and debugging tools
								
	UPDATED 20130827.1900 - BBR - added more print display formats

	UPDATED 20130817.1900 - BBR - completed integrating PBJ's words
								  added more print display formats
								
 	UPDATED 20130815.1800 - BBR - adapted for additional use with DS3231M
								  borrowed some routines from PBJ's 
								  MCP79410 code

	CREATED 20121223.0500 - BB Riley, Underhill Center, VT  

                        brianbr@wulfden.org

	TESTED  Tachyon Forth V21 20130811 Extend 20130814


	DS1307/DS3231 I2C ADDRESS IS 7 BITS = $68 left justified in the byte
    %110_1000 ==> %1101_0000 becomes $D0
}

: DSRTC.fth 	." DS1307/3231 RTC ToolKit - 20130908.1830" ;

IFNDEF rtcbuf
	BYTE	rtcbuf	8 ALLOT
}

1	CONSTANT	DS3231	\ 	Comment out one of these two chips, thus selecting the
\ 1	CONSTANT	DS1307	\	other chip. This will then link the correct speed I2C
						\	i/o primitives to RTC@, RTC!, and RTC!? ... slow for
						\	the DS1307 and fast for DS3231

$D0	CONSTANT	@rtc	\	both chips have the same I2C address

 0	CONSTANT	DSWRITE
 1	CONSTANT	DSREAD

 0	CONSTANT	DSSEC	\  seconds 0-59
 1	CONSTANT	DSMIN	\  minutes 0-59
 2	CONSTANT	DSHR	\  hours 0-23
 3	CONSTANT	DSDOW	\  day of week Sun=1 ... Sat=7
 4	CONSTANT	DSDAY	\  day of month
 5	CONSTANT	DSMON	\  month Jan=1 ... Dec=12
 6	CONSTANT	DSYR	\  year 0-99
 7	CONSTANT	DSCTL	\  control register

IFNDEF	DS3231
				\  although Tachyon Forth recognizes the DS1307 as  a 'fast' device, IT IS
				\  NOT!!! It needs to be run at 125K and NOT at 400K. Hence Peter's (S)low I2C 
				\  i/o is selected
				
:	RTC!	( byte -- )				SI2C! ;	
:	RTC!?	( byte -- flag )		SI2C!? ;
:	RTC@	( flag -- byte flag )	SI2C@  ;
}

IFNDEF DS1307
				\ DS3231 can keep up at 400K
				
:	RTC!	( byte -- )				I2C! ;	
:	RTC!?	( byte -- flag )		I2C!? ;
:	RTC@	( flag -- byte flag )	I2C@  ;
}

IFNDEF _EMITS 
:	_EMITS	( addr cnt -- )		ADO I C@ EMIT LOOP  ;
}

:	BCD>INT		( bcd -- int )	DUP $0F AND SWAP 4 SHR $0F AND #10 * + ;
:	INT>BCD		( int -- bcd )	DUP #10 / $10 * SWAP #10 MOD + ;
:	DAY$	( index -- addr )   3 * " ---SunMonTueWedThuFriSat" + ;
:	MONTH$	( index -- addr )   3 * " ---JanFebMarAprMayJunJulAugSepOctNovDec" + ;
:	LABEL$	( index -- addr )	3 * " SecMinHrsDoWDoMMonYrsCtl" +  ;


:	.MONTH	( index -- )		MONTH$ 3 _EMITS  ;
:	.DAY	( index -- )		DAY$ 3 _EMITS  ;
:	.LABEL	( index -- )		LABEL$ 3 _EMITS  ;

:	RTCMODE	( r/w -- flag )		\  sets slave read/write mode		
			I2CSTART @rtc OR RTC!? DROP ;
			
:	@RTCREG	( reg -- flag )		\  sets slave to register #
			DSWRITE RTCMODE RTC!? DROP ;
				
:	RTCREG!	( val reg -- flag )	 \  writes <val> to slave register #
			@RTCREG RTC!? DROP I2CSTOP ;
			
:	RDRTC	( -- )			\  reads date/time registers from slave to memory
			0 @RTCREG	DSREAD RTCMODE
			rtcbuf 8 ADO 0 RTC@ I C! LOOP
			1 RTC@  DROP  I2CSTOP
			;

\ ---------------- Time / Date / DayofWeek Setting -----------------

:	CTL!	( ctlmask -- ) DUP DSCTL rtcbuf + C! DSCTL  RTCREG! ;
							\ 

:	DAY!	( dayofweek -- ) DUP DSDOW rtcbuf + C! DSDOW  RTCREG! ;
							\ single digit 1-7  \ Sun = 1 ... Sat = 7

							\ TIME and DATE are entered in BCD format

:	TIME!	( $hh $mm $ss -- )	
					DUP	DSSEC  rtcbuf + C! DSSEC RTCREG!
					DUP	DSMIN  rtcbuf + C! DSMIN RTCREG!
					DUP	DSHR   rtcbuf + C! DSHR  RTCREG!
					;

:	DATE!	( $yy $mm $dd -- )	
					DUP	DSDAY  rtcbuf + C! DSDAY RTCREG!
					DUP	DSMON  rtcbuf + C! DSMON RTCREG!
					DUP	DSYR   rtcbuf + C! DSYR  RTCREG!
					;
\ ------------------- Date / Time Print Words  -------------------

:	.TIME	( -- )		\  print time string hh:mm:ss
			RDRTC  
			rtcbuf 3 ADO I C@ LOOP
			.BYTE ":" EMIT .BYTE ":" EMIT 	.BYTE
			;
			
:	.DATEI	( -- )		\  print international date string 20yy/mm/dd
			RDRTC  
			rtcbuf DSDAY + 3 ADO I C@ LOOP
			." 20" .BYTE "/" EMIT .BYTE "/" EMIT  .BYTE
			;

:	.DATE	( -- )		\  print date string  mm/dd/20yy
			RDRTC
			rtcbuf	5 + DUP C@ .BYTE "/" EMIT
		 			1-  DUP C@ .BYTE ." /20" 	
					2+  	C@ .BYTE
			;
			
:	.DATES	( -- )		\  print date string short mm/dd
			RDRTC
			rtcbuf	5 + DUP C@ .BYTE "/" EMIT
		 			1-  	C@ .BYTE 
			;
					
:	.DATEL	( -- )
			RDRTC		\ print long date  day, date month 20yy
			rtcbuf 3 + DUP C@	BCD>INT	.DAY ." , "	\  DoW
					1+ DUP C@   .BYTE  SPACE		\  Day
					1+ DUP C@ 	BCD>INT .MONTH 		\  Month	
					1+	   C@   ."  20" .BYTE		\  Year
			;
				
:	.TIMELOOP	( -- )
	CR CR
	BEGIN
		4 SPACE .TIME 4 SPACES #990 ms $0D EMIT 
	KEY? UNTIL
	DROP
	;
	
:	.RTCBUF		( -- )
		RDRTC
		CR ." ----------" CR
		0 8 ADO  I .LABEL  ."   = "  rtcbuf I + C@ .BYTE CR LOOP
		;

:	DRTC	( -- ) RDRTC
:	DBUF	( -- )  rtcbuf $20  DUMP  ;
:	.DT		( -- )	.DATE  SPACE .TIME ;        
:	.DTI	( -- )	.DATEI SPACE .TIME ;        
:	TL		( -- )	.TIMELOOP  ;
:	DT	CR .DATEL  CR .TIMELOOP  CR DBUF ;

: SETRTC
	0 @RTCREG

	$00 RTC!?	DROP	\  0 - secs
	$21 RTC!?	DROP	\  1 - mins
	$20 RTC!?	DROP	\  2 - hours
	$07 RTC!?	DROP	\  3 - day of week
	$07 RTC!?	DROP	\  4 - day of month
	$09 RTC!?	DROP	\  5 - month
	$13 RTC!?	DROP	\  6 - year
	$00 RTC!?	DROP	\  7 - control
	
	I2CSTOP
	
	CR
	;

]~

END

